@@ -265,9 +265,15 @@ bool QAVIFHandler::decode_one_frame()
         }
     }
 
+    uint32_t resultdepth = m_decoder->image->depth;
+    if (m_decoder->image->matrixCoefficients == 16 && m_decoder->image->yuvFormat == AVIF_PIXEL_FORMAT_YUV444 && resultdepth == 10) {
+        // 10-bit YCgCo-Re AVIF must be decoded to 8bit
+        resultdepth = 8;
+    }
+
     QImage::Format resultformat;
 
-    if (m_decoder->image->depth > 8) {
+    if (resultdepth > 8) {
         if (loadalpha) {
             resultformat = QImage::Format_RGBA64;
         } else {
@@ -293,9 +299,7 @@ bool QAVIFHandler::decode_one_frame()
         colorspace = QColorSpace::fromIccProfile(icc_data);
         if (!colorspace.isValid()) {
             qCWarning(LOG_AVIFPLUGIN, "AVIF image has Qt-unsupported or invalid ICC profile!");
-        }
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
-        else {
+        } else {
             if (colorspace.colorModel() == QColorSpace::ColorModel::Cmyk) {
                 qCWarning(LOG_AVIFPLUGIN, "CMYK ICC profile is not extected for AVIF, discarding the ICCprofile!");
                 colorspace = QColorSpace();
@@ -325,7 +329,6 @@ bool QAVIFHandler::decode_one_frame()
                 }
             }
         }
-#endif
     } else {
         float prim[8] = {0.64f, 0.33f, 0.3f, 0.6f, 0.15f, 0.06f, 0.3127f, 0.329f};
         // outPrimaries: rX, rY, gX, gY, bX, bY, wX, wY
@@ -360,14 +363,12 @@ bool QAVIFHandler::decode_one_frame()
         case 13:
             q_trc = QColorSpace::TransferFunction::SRgb;
             break;
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
         case 16: /* AVIF_TRANSFER_CHARACTERISTICS_PQ */
             q_trc = QColorSpace::TransferFunction::St2084;
             break;
         case 18: /* AVIF_TRANSFER_CHARACTERISTICS_HLG */
             q_trc = QColorSpace::TransferFunction::Hlg;
             break;
-#endif
         default:
             qCWarning(LOG_AVIFPLUGIN, "CICP colorPrimaries: %d, transferCharacteristics: %d\nThe colorspace is unsupported by this plug-in yet.",
                      m_decoder->image->colorPrimaries,
@@ -377,11 +378,9 @@ bool QAVIFHandler::decode_one_frame()
         }
 
         if (q_trc != QColorSpace::TransferFunction::Custom) { // we create new colorspace using Qt
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
             if (loadgray) {
                 colorspace = QColorSpace(whitePoint, q_trc, q_trc_gamma);
             } else {
-#endif
                 switch (m_decoder->image->colorPrimaries) {
                 /* AVIF_COLOR_PRIMARIES_BT709 */
                 case 0:
@@ -389,6 +388,9 @@ bool QAVIFHandler::decode_one_frame()
                 case 2: /* AVIF_COLOR_PRIMARIES_UNSPECIFIED */
                     colorspace = QColorSpace(QColorSpace::Primaries::SRgb, q_trc, q_trc_gamma);
                     break;
+                case 9:
+                    colorspace = QColorSpace(QColorSpace::Primaries::Bt2020, q_trc, q_trc_gamma);
+                    break;
                 /* AVIF_COLOR_PRIMARIES_SMPTE432 */
                 case 12:
                     colorspace = QColorSpace(QColorSpace::Primaries::DciP3D65, q_trc, q_trc_gamma);
@@ -397,9 +399,7 @@ bool QAVIFHandler::decode_one_frame()
                     colorspace = QColorSpace(whitePoint, redPoint, greenPoint, bluePoint, q_trc, q_trc_gamma);
                     break;
                 }
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
             }
-#endif
         }
 
         if (!colorspace.isValid()) {
@@ -414,7 +414,7 @@ bool QAVIFHandler::decode_one_frame()
     rgb.maxThreads = m_decoder->maxThreads;
 #endif
 
-    if (m_decoder->image->depth > 8) {
+    if (resultdepth > 8) {
         rgb.depth = 16;
         rgb.format = AVIF_RGB_FORMAT_RGBA;
 
@@ -743,22 +743,65 @@ bool QAVIFHandler::write(const QImage &image)
                 /* AVIF_TRANSFER_CHARACTERISTICS_LINEAR */
                 avif->transferCharacteristics = (avifTransferCharacteristics)8;
                 break;
+            case QColorSpace::TransferFunction::Gamma:
+                if (qAbs(tmpgrayimage.colorSpace().gamma() - 2.2f) < 0.1f) {
+                    /* AVIF_TRANSFER_CHARACTERISTICS_BT470M */
+                    avif->transferCharacteristics = (avifTransferCharacteristics)4;
+                } else if (qAbs(tmpgrayimage.colorSpace().gamma() - 2.8f) < 0.1f) {
+                    /* AVIF_TRANSFER_CHARACTERISTICS_BT470BG */
+                    avif->transferCharacteristics = (avifTransferCharacteristics)5;
+                }
+                break;
             case QColorSpace::TransferFunction::SRgb:
                 /* AVIF_TRANSFER_CHARACTERISTICS_SRGB */
                 avif->transferCharacteristics = (avifTransferCharacteristics)13;
                 break;
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
             case QColorSpace::TransferFunction::St2084:
                 avif->transferCharacteristics = (avifTransferCharacteristics)16;
                 break;
             case QColorSpace::TransferFunction::Hlg:
                 avif->transferCharacteristics = (avifTransferCharacteristics)18;
                 break;
-#endif
             default:
                 /* AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED */
                 break;
             }
+
+            if (avif->transferCharacteristics == 2) { // in case TransferFunction was not identified yet
+                if (tmpgrayimage.colorSpace().colorModel() == QColorSpace::ColorModel::Gray && lossless) {
+                    avif->colorPrimaries = (avifColorPrimaries)2;
+                    avif->matrixCoefficients = (avifMatrixCoefficients)6;
+
+                    QByteArray iccprofile_gray = tmpgrayimage.colorSpace().iccProfile();
+
+                    if (iccprofile_gray.size() > 0) {
+#if AVIF_VERSION >= 1000000
+                        res = avifImageSetProfileICC(avif, reinterpret_cast<const uint8_t *>(iccprofile_gray.constData()), iccprofile_gray.size());
+                        if (res != AVIF_RESULT_OK) {
+                            qCWarning(LOG_AVIFPLUGIN, "ERROR in avifImageSetProfileICC: %s", avifResultToString(res));
+                            return false;
+                        }
+#else
+                        avifImageSetProfileICC(avif, reinterpret_cast<const uint8_t *>(iccprofile_gray.constData()), iccprofile_gray.size());
+#endif
+                    }
+                } else { // convert to grayscale with SRgb
+                    tmpgrayimage.convertToColorSpace(QColorSpace(QPointF(0.3127f, 0.329f), QColorSpace::TransferFunction::SRgb), QImage::Format_Grayscale16);
+                    switch (tmpgrayimage.format()) {
+                    case QImage::Format_Grayscale8:
+                        save_depth = 8;
+                        break;
+                    case QImage::Format_Grayscale16:
+                        save_depth = 10;
+                        avif->transferCharacteristics = (avifTransferCharacteristics)13;
+                        break;
+                    default:
+                        qCWarning(LOG_AVIFPLUGIN, "Error saving Gray image");
+                        return false;
+                        break;
+                    }
+                }
+            }
         }
 
         if (save_depth > 8) { // QImage::Format_Grayscale16
@@ -799,7 +842,6 @@ bool QAVIFHandler::write(const QImage &image)
             }
         }
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 8, 0)
         QImage tmpcolorimage;
         auto cs = image.colorSpace();
         if (cs.isValid() && cs.colorModel() == QColorSpace::ColorModel::Cmyk && image.format() == QImage::Format_CMYK8888) {
@@ -814,9 +856,6 @@ bool QAVIFHandler::write(const QImage &image)
         } else {
             tmpcolorimage = image.convertToFormat(tmpformat);
         }
-#else
-        QImage tmpcolorimage = image.convertToFormat(tmpformat);
-#endif
 
         avifPixelFormat pixel_format = AVIF_PIXEL_FORMAT_YUV420;
         if (m_quality >= KIMG_AVIF_QUALITY_HIGH) {
@@ -847,6 +886,10 @@ bool QAVIFHandler::write(const QImage &image)
                 /* AVIF_MATRIX_COEFFICIENTS_CHROMA_DERIVED_NCL */
                 matrix_to_save = (avifMatrixCoefficients)12;
                 break;
+            case QColorSpace::Primaries::Bt2020:
+                primaries_to_save = (avifColorPrimaries)9;
+                matrix_to_save = (avifMatrixCoefficients)12;
+                break;
             default:
                 /* AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED */
                 primaries_to_save = (avifColorPrimaries)2;
@@ -876,14 +919,12 @@ bool QAVIFHandler::write(const QImage &image)
                 /* AVIF_TRANSFER_CHARACTERISTICS_SRGB */
                 transfer_to_save = (avifTransferCharacteristics)13;
                 break;
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
             case QColorSpace::TransferFunction::St2084:
                 transfer_to_save = (avifTransferCharacteristics)16;
                 break;
             case QColorSpace::TransferFunction::Hlg:
                 transfer_to_save = (avifTransferCharacteristics)18;
                 break;
-#endif
             default:
                 /* AVIF_TRANSFER_CHARACTERISTICS_UNSPECIFIED */
                 transfer_to_save = (avifTransferCharacteristics)2;
@@ -919,14 +960,12 @@ bool QAVIFHandler::write(const QImage &image)
                         case 5: // AVIF_TRANSFER_CHARACTERISTICS_BT470BG
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, 2.8f));
                             break;
-#if (QT_VERSION >= QT_VERSION_CHECK(6, 8, 0))
                         case 16:
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::St2084));
                             break;
                         case 18:
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::Hlg));
                             break;
-#endif
                         default: // AVIF_TRANSFER_CHARACTERISTICS_SRGB + any other
                             tmpcolorimage.convertToColorSpace(QColorSpace(QColorSpace::Primaries::SRgb, QColorSpace::TransferFunction::SRgb));
                             transfer_to_save = (avifTransferCharacteristics)13;
